'
' MEMORIZING COMBOBOX - Public Domain by Borje Hagsten, January 2002
' Shows how to create an ownerdrawn combobox than "remembers" last
' selected item and puts it at top of list, with double line divider.
' Compare MS Word's font name combo - probably the most (only?)
' "intelligent" code ever written by MS..
'
' Here set to keep up to 6 last selected items at top. Code to consider
' is under %WM_DRAWITEM and %WM_COMMAND, when CBCTL = 30.
' Free to use and modify as you whish. This code can also be used to do
' similar memorizing listbox. Then just change CB messages to LB messages.
' Tested and compiles fine with both PBDLL 6.0 and 6.1
'
#COMPILE EXE
#INCLUDE "WIN32API.INC"
DECLARE CALLBACK FUNCTION DlgCallback()

'
' PBMAIN - build dialog and controls
'
FUNCTION PBMAIN
  LOCAL ii AS LONG, hDlg AS LONG

  DIALOG NEW 0, "Memorizing Combobox",,, 130, 60, %WS_CAPTION OR %WS_SYSMENU TO hDlg
  CONTROL ADD BUTTON, hDlg, %IDCANCEL, "&Ok", 75, 42, 50, 14
  CONTROL ADD COMBOBOX, hDlg, 30,, 5, 5, 120, 140, _
                       %CBS_OWNERDRAWFIXED OR %CBS_HASSTRINGS OR %CBS_DROPDOWNLIST OR _
                       %WS_TABSTOP OR %WS_VSCROLL

  FOR ii = 1 TO 20 'add some items to combo
     COMBOBOX ADD hDlg, 30, "This is item" + STR$(ii)
  NEXT
  COMBOBOX SELECT hDlg, 30, 1 'and select first item

  DIALOG SHOW MODAL hDlg CALL DlgCallback
END FUNCTION

'
' MAIN DIALOG'S CALLBACK PROCEDURE
'
CALLBACK FUNCTION DlgCallback()
  SELECT CASE CBMSG
     CASE %WM_INITDIALOG
        LOCAL ii AS LONG, cSel AS LONG, tLen AS LONG, txt AS STRING, txt2 AS STRING
        STATIC memCount AS LONG

     CASE %WM_COMMAND
        SELECT CASE CBCTL
           CASE %IDCANCEL 'EXIT BUTTON CALLING..
              IF CBCTLMSG = %BN_CLICKED THEN DIALOG END CBHNDL

           CASE 30 'COMBOBOX CALLING..
              IF CBCTLMSG = %CBN_SELENDOK THEN                        'selection was made
                 CONTROL SEND CBHNDL, 30, %CB_GETCURSEL, 0, 0 TO cSel 'get sel number
                 COMBOBOX GET TEXT CBHNDL, 30 TO txt                  'get sel text

                 IF memCount THEN                                     'if anything previously selected
                    IF cSel < memCount THEN                           'if sel was made among mem items
                       CONTROL SEND CBHNDL, 30, %CB_DELETESTRING, cSel, 0 'remove sel item and decr mem counter
                       DECR memCount
                    ELSE                                              'if sel in "ordinary" list
                       FOR ii = 0 TO memCount - 1                     'look for item in mem list
                          CONTROL SEND CBHNDL, 30, %CB_GETLBTEXTLEN, ii, 0 TO tLen
                          txt2 = SPACE$(tLen + 1)
                          CONTROL SEND CBHNDL, 30, %CB_GETLBTEXT, ii, STRPTR(txt2)
                          txt2 = EXTRACT$(txt2, CHR$(0))
                          IF txt = txt2 THEN                                  'if in mem list
                             CONTROL SEND CBHNDL, 30, %CB_DELETESTRING, ii, 0 'delete mem list item
                             DECR memCount
                             EXIT FOR
                          END IF
                       NEXT
                    END IF
                 END IF

                 CONTROL SEND CBHNDL, 30, %CB_INSERTSTRING, 0, STRPTR(txt) 'insert sel item at top
                 IF memCount = 6 THEN                                      'if > 6 mem items
                    CONTROL SEND CBHNDL, 30, %CB_DELETESTRING, 6, 0        'delete item 7 (zero based)
                 END IF
                 memCount = MIN&(6, memCount + 1)             'increase counter
                 CONTROL SEND CBHNDL, 30, %CB_SETCURSEL, 0, 0 'first item is now same as selected
              END IF

        END SELECT

     CASE %WM_DRAWITEM
        IF CBWPARAM = 30 THEN 'our combobox id, in case other ownerdrawn stuff is added later..
           LOCAL lpdis  AS DRAWITEMSTRUCT PTR, zTxt AS ASCIIZ * %MAX_PATH
           lpdis = CBLPARAM
           IF @lpdis.itemID = &HFFFFFFFF& THEN EXIT FUNCTION 'if empty list..

           SELECT CASE @lpdis.itemAction
              CASE %ODA_DRAWENTIRE, %ODA_SELECT
                 'CLEAR BACKGROUND
                 IF (@lpdis.itemState AND %ODS_SELECTED) = 0 THEN                          'if not selected
                    CALL SETBKCOLOR(@lpdis.hDC, GETSYSCOLOR(%COLOR_WINDOW))                'text background
                    FILLRECT(@lpdis.hDC, @lpdis.rcItem, GETSYSCOLORBRUSH(%COLOR_WINDOW))   'clear background
                    CALL SETTEXTCOLOR(@lpdis.hDC, GETSYSCOLOR(%COLOR_WINDOWTEXT))          'text color
                 ELSE
                    CALL SETBKCOLOR(@lpdis.hDC, GETSYSCOLOR(%COLOR_HIGHLIGHT))             'text background
                    CALL SETTEXTCOLOR(@lpdis.hDC, GETSYSCOLOR(%COLOR_HIGHLIGHTTEXT))       'text color
                    FILLRECT(@lpdis.hDC, @lpdis.rcItem, GETSYSCOLORBRUSH(%COLOR_HIGHLIGHT))'clear background
                 END IF

                 'GET/DRAW TEXT
                 CALL SENDMESSAGE(GETDLGITEM(CBHNDL, 30), %CB_GETLBTEXT, @lpdis.itemID, VARPTR(zTxt))
                 IF (@lpdis.itemState AND &H1000) THEN 'if to draw %ODS_COMBOBOXEDIT (= &H1000)
                    @lpdis.rcItem.nLeft = 4            'set left margin a bit in
                 ELSE
                    @lpdis.rcItem.nLeft = 2            'set smaller left margin in list
                 END IF
                 DRAWTEXT @lpdis.hDC, BYVAL VARPTR(zTxt), LEN(zTxt), @lpdis.rcItem, %DT_LEFT OR %DT_SINGLELINE OR %DT_VCENTER

                 'DRAW DOUBLE LINES BETWEEN PREVIOUSLY SELECTED ITEMS AND "REAL" ITEMS
                 '(but do not draw any lines in edit box - %ODS_COMBOBOXEDIT)
                 IF memCount AND (@lpdis.itemState AND &H1000) = 0 THEN
                    IF @lpdis.itemID = memCount THEN
                       MOVETO @lpdis.hDC, 0, @lpdis.rcItem.nTop - 3
                       LINETO @lpdis.hDC, @lpdis.rcItem.nRight, @lpdis.rcItem.nTop - 3
                       MOVETO @lpdis.hDC, 0, @lpdis.rcItem.nTop - 1
                       LINETO @lpdis.hDC, @lpdis.rcItem.nRight, @lpdis.rcItem.nTop - 1
                    ELSEIF @lpdis.itemID = memCount - 1 THEN
                       MOVETO @lpdis.hDC, 0, @lpdis.rcItem.nBottom - 3
                       LINETO @lpdis.hDC, @lpdis.rcItem.nRight, @lpdis.rcItem.nBottom - 3
                       MOVETO @lpdis.hDC, 0, @lpdis.rcItem.nBottom - 1
                       LINETO @lpdis.hDC, @lpdis.rcItem.nRight, @lpdis.rcItem.nBottom - 1
                    END IF
                 END IF

                 'SELECTED ITEM
                 IF (@lpdis.itemState AND %ODS_SELECTED) THEN      'if selected
                    CALL DRAWFOCUSRECT(@lpdis.hDC, @lpdis.rcItem)  'draw focus rect in list
                 END IF
                 FUNCTION = %TRUE : EXIT FUNCTION 'DefWindowProc handles focus rect in edit box..
           END SELECT
        END IF

   END SELECT
END FUNCTION
